################################################################################
## copyright notice:
################################################################################
## copyright (c) 2014 by
##
## CESYS GmbH / GERMANY
##
## http://www.cesys.com
################################################################################
## author(s):
################################################################################
## MAnfred Kraus <manfred.kraus@cesys.com>
################################################################################
## license:
################################################################################
## THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH CESYS PRODUCTS ONLY!
## YOU ARE NOT ALLOWED TO MODIFY AND DISTRIBUTE OR USE IT WITH ANY OTHER
## HARDWARE, SOFTWARE OR ANY OTHER KIND OF ASIC OR PROGRAMMABLE LOGIC DESIGN
## WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDER!
################################################################################
## disclaimer of warranty:
################################################################################
## THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
## THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
## PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
## EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
## THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
## YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
## NECESSARY SERVICING, REPAIR OR CORRECTION.
##
## IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
## ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
## USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
## DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
## PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
## SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
## PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
## THE POSSIBILITY OF SUCH DAMAGES.
################################################################################

# usage:
# call from Xilinx ISE TCL Shell (do not doubleclick to run)
# % source efm02_soc_xc6slx45_bitgen.tcl
# "Generate Programming File" process from GUI

set host_sec [clock seconds]
set host_date [clock format $host_sec -format {%d. %b. %Y}]
set host_time [clock format $host_sec -format %H:%M:%S]

# function for extracting constants from vhdl source code
proc vhd_extract_const {src_dir} {
  
  # valid vhdl formats for defining constants:
  # address info =>
  # constant MYADDR : std_logic_vector(...) := x"CAFE_BABE";
  # bit flag =>
  # constant MYFLAG : natural := 12;
  # bit field =>
  # subtype MYFIELD is natural range 31 downto 28;
  # result:
  # #define MYADDR 0xCAFEBABE
  # #define MYFLAG (0x1<<12)
  # #define MYFIELD_POS 28
  # #define MYFIELD (0xF<<MYFIELD_POS)
  
  # searching vhdl packages
  #set vhd_pkgs [glob -nocomplain "[set dnc $src_dir]/*_pkg.vhd"]
  #set vhd_pkgs $src_dir
  set vhd_pkgs "
    $src_dir/efm02_soc_pkg.vhd
    $src_dir/wb_ma_fx3_pkg.vhd
    $src_dir/wb_sl_bram_pkg.vhd
    $src_dir/wb_sl_flash_pkg.vhd
    $src_dir/wb_sl_gpio_pkg.vhd
    $src_dir/wb_sl_mcb_pkg.vhd
  "
  
  set memmap {}
  
  foreach pkg $vhd_pkgs {
    
    # reading package contents from file
    set pkg_file [open $pkg r]
    set pkg_content [read -nonewline $pkg_file]
    close $pkg_file
    #puts $pkg_content
    
    # defining start and stop tokens
    set parse_on_token "--\\s*pragma\\s*CONST_EXTRACT_ON"
    set parse_off_token "--\\s*pragma\\s*CONST_EXTRACT_OFF"
    
    # returning text between tokens
    if {[regexp -- "$parse_on_token.*$parse_off_token" $pkg_content constants]} {
      # removing tokens
      regsub -- $parse_on_token $constants "" constants
      regsub -- $parse_off_token $constants "" constants
      
      # splitting vhdl constructs
      set constants [split $constants ";"]
      
      foreach constant $constants {
        set constant [string trim $constant]
        if {$constant != ""} {
          # vhdl constants for addresses and flags
          if {[string equal -nocase [string range $constant 0 7] "constant"]} {
            set constant [string range $constant 8 end]
            set constant [split $constant ":"]
            set constant_name [string trim [lindex $constant 0]]
            set constant_val [lindex $constant 2]
            regsub -all -- "=|x|_|\"" $constant_val "" constant_val
            set constant_val [string trim $constant_val]
            # flag
            if {[string equal -nocase [string trim [lindex $constant 1]] "natural"]} {
              set constant_val "(0x1<<$constant_val)"
            # address
            } else {
              set constant_val "0x$constant_val"
            }
          # vhdl ranges for fields
          } else {
            set constant [string trim [string range $constant 7 end]]
            set constant [string trim [split $constant]]
            set constant_name [lindex $constant 0]
            lappend memmap "[set dnc $constant_name]_POS [lindex $constant end]"
            set field 0
            for {set i 0} {$i <= [expr [lindex $constant end-2]-[lindex $constant end]]} {incr i} {
              set field [expr $field|(0x1<<$i)]
            }
            set field [format %X $field]
            set constant_val "(0x$field<<[set dnc $constant_name]_POS)"
          }
          #puts "$constant_name $constant_val"
          lappend memmap "$constant_name $constant_val"
        }
      }
    }
  }
  
  return $memmap
}

proc incr_version {VERSION_MAJOR VERSION_MINOR VERSION_REV} {
  
  global host_date
  global host_time
  
  # reading and incrementing current build number from current vhdl package
  set pf [open "../src/efm02_xc6slx45_build_pkg.vhd" r]
  set pf_content [read -nonewline $pf]
  close $pf
  set VERSION_BUILD [expr [string range $pf_content [expr [string first ":=" $pf_content]+2] [expr [string first ";" $pf_content]-1]]]
  incr VERSION_BUILD

  # creating new vhdl package containing updated build number
  set pf [open "../src/efm02_xc6slx45_build_pkg.vhd" w]
  puts -nonewline $pf \
"package build_pkg is
  constant BUILD : positive := [set dnc $VERSION_BUILD];
end build_pkg;"
  close $pf
  
  set VERSION "[format %02X $VERSION_MAJOR][format %02X $VERSION_MINOR][format %02X $VERSION_REV]"
  set VERSION_STR "[set dnc $VERSION_MAJOR]v[set dnc $VERSION_MINOR]r[set dnc $VERSION_REV]b[set dnc $VERSION_BUILD]"
  
  # creating vhdl package containing version
  set pf [open "../src/efm02_xc6slx45_version_pkg.vhd" w]
  puts -nonewline $pf \
"-- created $host_date - $host_time
library IEEE;
use IEEE.STD_LOGIC_1164.all;


package version_pkg is
  constant VERSION : std_logic_vector(31 downto 8) := x\"[set dnc $VERSION]\";
end version_pkg;"
  close $pf
  
  return "$VERSION[format %02X $VERSION_BUILD] $VERSION_STR"

}

proc create_sw_api_files {filename version} {
  
  global host_date
  global host_time
  
  set h [open "$filename.h" w]
  set py [open "$filename.py" w]
  puts $h "/* created $host_date - $host_time */"
  puts $h "const uint32 EFM02_SOC_XC6SLX45_FPGA_DESIGN_VERSION = 0x$version;"
  puts $py "# created $host_date - $host_time"
  puts $py "EFM02_SOC_XC6SLX45_FPGA_DESIGN_VERSION = 0x$version"
  foreach const_val [vhd_extract_const "../src"] {
    #puts $h "#define $const_val"
    puts $h "const uint32 [lindex $const_val 0] = [lindex $const_val 1];"
    puts $py "[lindex $const_val 0] = [lindex $const_val 1]"      
  }
  close $py
  close $h
  
}

# script main entry point
#########################


if {[process run "Check Syntax"]} {
  
  set version [incr_version 0 0 0]
    
  # running synthesis
  if {[process run "Generate Programming File"]} {
      
    # software constant files
    set sw_files_name "efm02_soc_xc6slx45_fpga_consts"
    create_sw_api_files $sw_files_name [lindex $version 0]
    
    # copying files to working directory
    #set wd "w:/pss"
    set wd "s:/projects/CESYS GmbH/EFM02"
    file copy -force "$sw_files_name.py" "$wd/scripts/$sw_files_name.py" 
    file copy -force "$sw_files_name.h" "$wd/sw/$sw_files_name.h" 
    file copy -force "efm02_soc_top_xc6slx45.bin" "$wd/scripts/efm02_soc_top_xc6slx45.bin" 
    file copy -force "efm02_soc_top_xc6slx45.bin" "$wd/work/efm02_soc_top_xc6slx45.bin"
    
    # creating a project snapshot
    # exec -- "c:/Xilinx/14.7/ISE_DS/ISE/bin/nt/zip.exe" -r "snap/efm02_soc_snap[lindex $version 1].zip" \
    # "efm02_soc_xc6slx45_fpga_consts.h" \
    # "efm02_soc_xc6slx45_fpga_consts.py" \
    # "efm02_soc_top_xc6slx45.bit" \
    # "efm02_soc_top_xc6slx45.bin" \
    # "pss_blvl.ucf" \
    # "efm02_soc_xc6slx45_bitgen.tcl" \
    # "efm02_soc_SLX45.xise" \
    # "../src" \
    # "../ipcore_dir" 
    
    # backup snapshot
    #file copy -force "snap/efm02_soc_snap[lindex $version 1].zip" "p:/cesys/efm02/hdl_design/snap/efm02_soc_snap.zip"
  }
  
}
