----------------------------------------------------------------------------------
-- Description:    	This is a VHDL top-level file for the CESYS EFM-02 module
-- Design Name:		efm02_soc (system-on-chip) example design
--
-- Company: 		CESYS GmbH
-- Author: 			Manfred Kraus <manfred.kraus@cesys.com>
-- 
-- File Name       	efm02_soc_top_XC6SLX150.vhd
-- Create Date:    	14:07:59 08/06/2013 
-- Module Name:    	efm02_toplevel - RTL 
-- Target Devices: 	XC6SLX45-3FGG484, XC6SLX100-3FGG484, XC6SLX150-3FGG484, 
-- Tool versions:  	ISE Project Navigator 14.6
---------------------------------------------------------------------------------------------------
-- Copyright © 2013 by CESYS GmbH, Germany. All rights are reserved. 
-- Unauthorized duplication of this document, in whole or in part, by any means is prohibited
-- without the prior written permission of CESYS GmbH, Germany.
-- 
-- Although CESYS GmbH believes that the information included in this publication is correct
-- as of the date of publication, it is provided "as is" without any support or guarantee.
-- CESYS reserves the right to make changes at any time without notice.
-- 
-- All information in this document may only be published by CESYS GmbH, Germany.
---------------------------------------------------------------------------------------------------
--    
---------------------------------------------------------------------------------------------------
-- File history:
--
-- Version | Date       | Author           | Remarks
-- ------------------------------------------------------------------------------------------------
-- 1.0     | 06.08.2013 | M. Kraus         | First release
---------------------------------------------------------------------------------------------------

library ieee;
    use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use ieee.math_real.all;
    
library unisim;
use unisim.vcomponents.all;

use work.efm02_soc_pkg.all;
use work.wishbone_pkg.all;
use work.wb_ma_fx3_pkg.all;
use work.wb_sl_gpio_pkg.all;
use work.wb_sl_bram_pkg.all;
use work.wb_sl_flash_pkg.all;
use work.wb_sl_mcb_pkg.all;


entity efm02_soc_top_XC6SLX150 is
  generic(
	C3_P0_MASK_SIZE			  : integer := 4;
	C3_P0_DATA_PORT_SIZE	  : integer := 32;
	C3_P1_MASK_SIZE			  : integer := 4;
	C3_P1_DATA_PORT_SIZE	  : integer := 32;
	C3_MEMCLK_PERIOD		  : integer := 3000;
	C3_RST_ACT_LOW			  : integer := 0;
	C3_INPUT_CLK_TYPE		  : string := "SINGLE_ENDED";
	C3_CALIB_SOFT_IP		  : string := "TRUE";
	C3_SIMULATION			  : string := "FALSE";
	DEBUG_EN				  : integer := 0;
	C3_MEM_ADDR_ORDER		  : string := "ROW_BANK_COLUMN";
	C3_NUM_DQ_PINS			  : integer := 16;
	C3_MEM_ADDR_WIDTH		  : integer := 14;
	C3_MEM_BANKADDR_WIDTH	  : integer := 3
  );
  port (
	pin_fx3_flag_i   : in	   std_logic_vector(3 downto 0);
	pin_fx3_slcs_n_o   : out   std_logic;
	pin_fx3_sloe_n_o   : out   std_logic;
	pin_fx3_slrd_n_o   : out   std_logic;
	pin_fx3_slwr_n_o   : out   std_logic;
	pin_fx3_pktend_n_o : out   std_logic;
	pin_fx3_fifoadr_o  : out   std_logic_vector(1 downto 0);
	pin_fx3_fd_io	   : inout std_logic_vector(31 downto 0);


	pin_cfg_flash_s_n_o : out std_logic;
	pin_cfg_flash_c_o	: out std_logic;
	pin_cfg_flash_d_o	: out std_logic; 
	pin_cfg_flash_q_i	: in  std_logic;

   	pin_gpio : inout std_logic_vector(190 downto 0);

	pin_user_led_o : out std_logic;
	
	mcb3_dram_dq							: inout	 std_logic_vector(C3_NUM_DQ_PINS-1 downto 0);
	mcb3_dram_a								: out std_logic_vector(C3_MEM_ADDR_WIDTH-1 downto 0);
	mcb3_dram_ba							: out std_logic_vector(C3_MEM_BANKADDR_WIDTH-1 downto 0);
	mcb3_dram_ras_n							: out std_logic;
	mcb3_dram_cas_n							: out std_logic;
	mcb3_dram_we_n							: out std_logic;
	mcb3_dram_odt							: out std_logic;
	mcb3_dram_cke							: out std_logic;
	mcb3_dram_dm							: out std_logic;
	mcb3_dram_udqs							: inout	 std_logic;
	mcb3_dram_udqs_n						: inout	 std_logic;
	mcb3_rzq								: inout	 std_logic;
	mcb3_zio								: inout	 std_logic;
	mcb3_dram_udm							: out std_logic;
	mcb3_dram_dqs							: inout	 std_logic;
	mcb3_dram_dqs_n							: inout	 std_logic;
	mcb3_dram_ck							: out std_logic;
	mcb3_dram_ck_n							: out std_logic;

	pin_clk100_i	: in std_logic; -- external oscillator (default: 100 MHz)
	pin_gpifclk_o	: out std_logic; -- source of FX-3 <--> FPGA clock network on PCB
	pin_gpifclk_i	: in std_logic; -- feedback of FX-3 <--> FPGA clock network on PCB

	pin_sysreset_i : in std_logic
  );
end efm02_soc_top_XC6SLX150;


architecture rtl of efm02_soc_top_XC6SLX150 is
  
  signal cfg_flash_c : std_logic := '0';

  signal syscon	  : rec_syscon_port		:= SYSCON_DEFAULT;
  signal intercon : rec_intercon_signal := INTERCON_DEFAULT;

  signal gpio_output : std_logic_vector(255 downto 0) := (others => '0');
  signal gpio_oe	 : std_logic_vector(255 downto 0) := (others => INPUT);
  signal gpio_input	 : std_logic_vector(255 downto 0) := (others => '0');

	signal clk100_i				: std_logic;
	signal ledclk				: std_logic;
  
	signal clk_gpif_n			: std_logic;
	signal clk_gpif				: std_logic;
	signal sys_clk  			: std_logic := '0';
    signal fx3_plclk  			: std_logic := '0';

	signal sys_clk_locked	    : std_logic := '0';
	
	signal gpifclk_o: std_logic := '0';
	signal gpifclk_tristate: std_logic := '0';
	
--	signal Freshly_configured_n : std_logic := '0';
--    signal FPGA_Reset: std_logic := '1';
    signal sysreset_i : std_logic := '1';
    

  	constant clk_frequency_Hz			: real := 100.0e+6; -- 100 MHz
	constant BlinkingLedFrequency		: real := 4.0;  -- 2 Hz 
	constant BlinkingLedToggleCycles 	: positive := integer(ceil(clk_frequency_Hz / (2.0 * BlinkingLedFrequency)));

	signal BlinkingLed				: std_logic := '0';   
	signal ManualLedControl			: std_logic := '0';   
	signal ManualLedState			: std_logic := '0';   

  signal c3_calib_done : std_logic := '0';
  signal c3_rst0 : std_logic := '0';
	
  signal c3_p0_cmd_clk		 : std_logic := '0';
  signal c3_p0_cmd_en		 : std_logic := '0';
  signal c3_p0_cmd_instr	 : std_logic_vector(2 downto 0) := (others => '0');
  signal c3_p0_cmd_bl		 : std_logic_vector(5 downto 0) := (others => '0');
  signal c3_p0_cmd_byte_addr : std_logic_vector(29 downto 0) := (others => '0');
  signal c3_p0_cmd_empty	 : std_logic := '0';
  signal c3_p0_cmd_full		 : std_logic := '0';
  signal c3_p0_wr_clk		 : std_logic := '0';
  signal c3_p0_wr_en		 : std_logic := '0';
  signal c3_p0_wr_mask		 : std_logic_vector(C3_P0_MASK_SIZE - 1 downto 0) := (others => '0');
  signal c3_p0_wr_data		 : std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0) := (others => '0');
  signal c3_p0_wr_full		 : std_logic := '0';
  signal c3_p0_wr_empty		 : std_logic := '0';
  signal c3_p0_wr_count		 : std_logic_vector(6 downto 0) := (others => '0');
  signal c3_p0_wr_underrun	 : std_logic := '0';
  signal c3_p0_wr_error		 : std_logic := '0';
  signal c3_p0_rd_clk		 : std_logic := '0';
  signal c3_p0_rd_en		 : std_logic := '0';
  signal c3_p0_rd_data		 : std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0) := (others => '0');
  signal c3_p0_rd_full		 : std_logic := '0';
  signal c3_p0_rd_empty		 : std_logic := '0';
  signal c3_p0_rd_count		 : std_logic_vector(6 downto 0) := (others => '0');
  signal c3_p0_rd_overflow	 : std_logic := '0';
  signal c3_p0_rd_error		 : std_logic := '0';

  
  component mcb_ddr2 is
 generic(
	C3_P0_MASK_SIZE			  : integer := 4;
	C3_P0_DATA_PORT_SIZE	  : integer := 32;
	C3_P1_MASK_SIZE			  : integer := 4;
	C3_P1_DATA_PORT_SIZE	  : integer := 32;
	C3_MEMCLK_PERIOD		  : integer := 3000;
	C3_RST_ACT_LOW			  : integer := 0;
	C3_INPUT_CLK_TYPE		  : string := "SINGLE_ENDED";
	C3_CALIB_SOFT_IP		  : string := "TRUE";
	C3_SIMULATION			  : string := "FALSE";
	DEBUG_EN				  : integer := 0;
	C3_MEM_ADDR_ORDER		  : string := "ROW_BANK_COLUMN";
	C3_NUM_DQ_PINS			  : integer := 16;
	C3_MEM_ADDR_WIDTH		  : integer := 14;
	C3_MEM_BANKADDR_WIDTH	  : integer := 3
);
	port (
   mcb3_dram_dq							   : inout	std_logic_vector(C3_NUM_DQ_PINS-1 downto 0);
   mcb3_dram_a							   : out std_logic_vector(C3_MEM_ADDR_WIDTH-1 downto 0);
   mcb3_dram_ba							   : out std_logic_vector(C3_MEM_BANKADDR_WIDTH-1 downto 0);
   mcb3_dram_ras_n						   : out std_logic;
   mcb3_dram_cas_n						   : out std_logic;
   mcb3_dram_we_n						   : out std_logic;
   mcb3_dram_odt						   : out std_logic;
   mcb3_dram_cke						   : out std_logic;
   mcb3_dram_dm							   : out std_logic;
   mcb3_dram_udqs						   : inout	std_logic;
   mcb3_dram_udqs_n						   : inout	std_logic;
   mcb3_rzq								   : inout	std_logic;
   mcb3_zio								   : inout	std_logic;
   mcb3_dram_udm						   : out std_logic;
   c3_sys_clk							   : in	 std_logic;
   c3_sys_rst_i							   : in	 std_logic;
   c3_calib_done						   : out std_logic;
   c3_clk0								   : out std_logic;
   c3_rst0								   : out std_logic;
   mcb3_dram_dqs						   : inout	std_logic;
   mcb3_dram_dqs_n						   : inout	std_logic;
   mcb3_dram_ck							   : out std_logic;
   mcb3_dram_ck_n						   : out std_logic;
   c3_p0_cmd_clk						   : in std_logic;
   c3_p0_cmd_en							   : in std_logic;
   c3_p0_cmd_instr						   : in std_logic_vector(2 downto 0);
   c3_p0_cmd_bl							   : in std_logic_vector(5 downto 0);
   c3_p0_cmd_byte_addr					   : in std_logic_vector(29 downto 0);
   c3_p0_cmd_empty						   : out std_logic;
   c3_p0_cmd_full						   : out std_logic;
   c3_p0_wr_clk							   : in std_logic;
   c3_p0_wr_en							   : in std_logic;
   c3_p0_wr_mask						   : in std_logic_vector(C3_P0_MASK_SIZE - 1 downto 0);
   c3_p0_wr_data						   : in std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
   c3_p0_wr_full						   : out std_logic;
   c3_p0_wr_empty						   : out std_logic;
   c3_p0_wr_count						   : out std_logic_vector(6 downto 0);
   c3_p0_wr_underrun					   : out std_logic;
   c3_p0_wr_error						   : out std_logic;
   c3_p0_rd_clk							   : in std_logic;
   c3_p0_rd_en							   : in std_logic;
   c3_p0_rd_data						   : out std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
   c3_p0_rd_full						   : out std_logic;
   c3_p0_rd_empty						   : out std_logic;
   c3_p0_rd_count						   : out std_logic_vector(6 downto 0);
   c3_p0_rd_overflow					   : out std_logic;
   c3_p0_rd_error						   : out std_logic
);
end component;
component clock_alignment
port
 (-- Clock in ports
  CLK_IN1           : in     std_logic;
  -- Clock out ports
  CLK_OUT1          : out    std_logic;
  -- Status and control signals
  RESET             : in     std_logic;
  LOCKED            : out    std_logic
 );
end component;

component gpif_clock
port
 (-- Clock in ports
  CLK_IN1           : in     std_logic;
  -- Clock out ports
  CLK_OUT1          : out    std_logic;
  CLK_OUT2          : out    std_logic;
  CLK_OUT3          : out    std_logic;
  CLK_OUT4          : out    std_logic
 );
end component;


begin  -- rtl

    pin_cfg_flash_c_o <= cfg_flash_c when syscon.rst = '0' else 'Z';    
	syscon.clk <= sys_clk;
    
---------------------------------------------------
--  Clock management and GPIF clock generation
---------------------------------------------------

-- IBUF the incoming clock (tip: this ibuffed clock can be used as clock for mcb)
  IBUFG_inst_clk100 : IBUFG
  generic map (
	 IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
	 IOSTANDARD => "DEFAULT")
  port map (
	 O => clk100_i, -- Clock buffer output
	 I => pin_clk100_i	-- Clock buffer input (connect directly to top-level port)
  );

      
gpif_clock_generator : gpif_clock
  port map
   (-- Clock in ports
    CLK_IN1 => clk100_i,
    --Clock out ports
    CLK_OUT1 => clk_gpif,   -- 100.000 MHz
    CLK_OUT2 => open,    -- 150.000 MHz
--    CLK_OUT2 => sys_clk,    -- 150.000 MHz
    CLK_OUT3 => open,        -- 180.000 MHz
    CLK_OUT4 => sys_clk        -- 50.000 MHz
    );

-- soc demo needs not run at maximum frequency.
-- For benchmarking use performance demo.
--sys_clk <= clk_gpif;


-- produce a 180° version of the clock	
	clk_gpif_n <= not clk_gpif;

-- use the clock and its 180° version with a ODDR2 to output the clock on a regular I/O pin.	
  inst_ODDR2_clk : ODDR2
  generic map(
	DDR_ALIGNMENT => "C0",			-- Sets output alignment to "NONE", "C0", "C1" : Input data D0, D1 is alligned to C0 = i_clk
	INIT		  => '0',			-- Sets initial state of the Q output to '0' or '1'
	SRTYPE		  => "ASYNC")		-- Specifies "SYNC" or "ASYNC" set/reset
  port map (						
	Q			  => gpifclk_o,	-- 1-bit output data
	C0			  => clk_gpif,		-- 1-bit clock input
	C1			  => clk_gpif_n,	-- 1-bit clock input
	CE			  => '1',			-- 1-bit clock enable input
	D0			  => '1',			-- 1-bit data input (associated with C0)
	D1			  => '0',			-- 1-bit data input (associated with C1)
	R			  => '0',			-- 1-bit reset input
	S			  => '0'			-- 1-bit set input
  );

pin_gpifclk_o <= gpifclk_o;

  
-------------------------------------------------------
-- produce sys_clock to drive the user fpga-fabric  
-------------------------------------------------------   
    inst_clock_alignment : clock_alignment
	 port map
   (-- Clock in ports
	CLK_IN1 => pin_gpifclk_i,
	CLK_OUT1 => fx3_plclk,
	RESET	=> '0',
	LOCKED => sys_clk_locked
    );    
   
   
  process(syscon)
  begin
    if syscon.clk'event and syscon.clk = '1' then
        sysreset_i <= pin_sysreset_i; -- sync external Reset signal
        syscon.rst <= sysreset_i or (not sys_clk_locked) or c3_rst0;
    end if;
  end process;
    
  
  	-----------------------------------------------------------------------------------------------
	-- Blink the Led !
	-----------------------------------------------------------------------------------------------

    ledclk <= clk_gpif;
    
	process (ledclk)
		variable cnt: INTEGER RANGE 0 to BlinkingLedToggleCycles;
	begin
		if rising_edge (ledclk) then
            -- if (syscon.rst = '1') then
                -- BlinkingLed <= '0';
                -- cnt := 0;
            -- else    
                if (cnt = 0) then
                    cnt := BlinkingLedToggleCycles ;
                    BlinkingLed <= not BlinkingLed;
                else
                    cnt := cnt - 1;
                end if;
--            end if;
		end if;
	end process;

	pin_user_led_o <= BlinkingLed xor c3_calib_done when ManualLedControl = '0' else ManualLedState; 
 -- pin_user_led_o <= c3_calib_done and (not (c3_p0_wr_underrun or c3_p0_wr_error or c3_p0_rd_overflow or c3_p0_rd_error));
 
  process(gpio_oe, gpio_output)
  begin
	for i in 0 to 190 loop
	  if gpio_oe(i) = OUTPUT then
		pin_gpio(i) <= gpio_output(i);
	  else
		pin_gpio(i) <= 'Z';
	  end if;
	end loop;  -- i
  end process;
  
  gpio_input(190 downto 0)	<= pin_gpio;
  gpio_input(255 downto 191) <= gpio_output(255 downto 191);

  ManualLedControl <= gpio_output(254);   
  ManualLedState <= gpio_output(255);   
  inst_wb_intercon : wb_intercon
	generic map (
	  NR_OF_DBGPORTS => 1
	)
	port map (
	  syscon_i => syscon,

	  master_i => intercon.masters.master,
	  slave_o  => intercon.masters.slave,

	  master_o => intercon.slaves.master,
	  slave_i  => intercon.slaves.slave,

	  debug => open
	);
	
  inst_wb_ma_fx3 : wb_ma_fx3
	generic map (
	  NR_OF_DBGPORTS => 16
	)
	port map (
	  syscon_i => syscon,
	  master_o => intercon.masters.master(MA_fx3_ID),
	  slave_i  => intercon.masters.slave(MA_fx3_ID),
      
      pin_sysreset_i => pin_sysreset_i,
      
      fx3_plclk      => fx3_plclk,
	  fx3_flag_i	 => pin_fx3_flag_i,
	  fx3_slcs_n_o   => pin_fx3_slcs_n_o,
	  fx3_sloe_n_o	 => pin_fx3_sloe_n_o,
	  fx3_slrd_n_o	 => pin_fx3_slrd_n_o,
	  fx3_slwr_n_o	 => pin_fx3_slwr_n_o,
	  fx3_pktend_n_o => pin_fx3_pktend_n_o,
	  fx3_fifoadr_o	 => pin_fx3_fifoadr_o,
	  fx3_fd_io		 => pin_fx3_fd_io,

	  debug => open
	);

  cfg_wb_sl_flash : wb_sl_flash
	generic map (
	  NR_OF_DBGPORTS => 1
	)
	port map (
	  syscon_i => syscon,
	  master_i => intercon.slaves.master(SL_CFG_FLASH_ID),
	  slave_o  => intercon.slaves.slave(SL_CFG_FLASH_ID),

	  flash_s_n_o	 => pin_cfg_flash_s_n_o,
	  flash_c_o		 => cfg_flash_c,
	  flash_hold_n_o => open,
	  flash_w_n_o	 => open,
	  flash_d_o		 => pin_cfg_flash_d_o,
	  flash_q_i		 => pin_cfg_flash_q_i,

	  debug => open
	);

  inst_wb_sl_gpio : wb_sl_gpio
	generic map (
	  NR_OF_DBGPORTS => 1
	)
	port map (
	  syscon_i		=> syscon,
	  master_i		=> intercon.slaves.master(SL_GPIO_ID),
	  slave_o		=> intercon.slaves.slave(SL_GPIO_ID),
	  
	  gpio_output_o => gpio_output,
	  gpio_oe_o		=> gpio_oe,
	  gpio_input_i	=> gpio_input,
	  
	  debug => open
	);

 
  inst_wb_sl_bram : wb_sl_bram
	generic map (
	  NR_OF_DBGPORTS => 1
	)
	port map (
	  syscon_i => syscon,
	  master_i => intercon.slaves.master(SL_BRAM_ID),
	  slave_o  => intercon.slaves.slave(SL_BRAM_ID),
	  
	  debug => open
	);

  inst_wb_sl_mcb : wb_sl_mcb
	generic map (
	  NR_OF_DBGPORTS => 1
	)
	port map (
	  syscon_i => syscon,
	  master_i => intercon.slaves.master(SL_MCB_ID),
	  slave_o  => intercon.slaves.slave(SL_MCB_ID),

	  cmd_clk		=> c3_p0_cmd_clk,
	  cmd_en		=> c3_p0_cmd_en,
	  cmd_instr		=> c3_p0_cmd_instr,
	  cmd_bl		=> c3_p0_cmd_bl,
	  cmd_byte_addr => c3_p0_cmd_byte_addr,
	  cmd_empty		=> c3_p0_cmd_empty,
	  cmd_full		=> c3_p0_cmd_full,
	  wr_clk		=> c3_p0_wr_clk,
	  wr_en			=> c3_p0_wr_en,
	  wr_mask		=> c3_p0_wr_mask,
	  wr_data		=> c3_p0_wr_data,
	  wr_full		=> c3_p0_wr_full,
	  wr_empty		=> c3_p0_wr_empty,
	  wr_count		=> c3_p0_wr_count,
	  wr_underrun	=> c3_p0_wr_underrun,
	  wr_error		=> c3_p0_wr_error,
	  rd_clk		=> c3_p0_rd_clk,
	  rd_en			=> c3_p0_rd_en,
	  rd_data		=> c3_p0_rd_data,
	  rd_full		=> c3_p0_rd_full,
	  rd_empty		=> c3_p0_rd_empty,
	  rd_count		=> c3_p0_rd_count,
	  rd_overflow	=> c3_p0_rd_overflow,
	  rd_error		=> c3_p0_rd_error,

	  debug => open
	);

	 inst_mcb_ddr2 : mcb_ddr2
	generic map (
	C3_P0_MASK_SIZE => C3_P0_MASK_SIZE,
	C3_P0_DATA_PORT_SIZE => C3_P0_DATA_PORT_SIZE,
	C3_P1_MASK_SIZE => C3_P1_MASK_SIZE,
	C3_P1_DATA_PORT_SIZE => C3_P1_DATA_PORT_SIZE,
	C3_MEMCLK_PERIOD => C3_MEMCLK_PERIOD,
	C3_RST_ACT_LOW => C3_RST_ACT_LOW,
	C3_INPUT_CLK_TYPE => C3_INPUT_CLK_TYPE,
	C3_CALIB_SOFT_IP => C3_CALIB_SOFT_IP,
	C3_SIMULATION => C3_SIMULATION,
	DEBUG_EN => DEBUG_EN,
	C3_MEM_ADDR_ORDER => C3_MEM_ADDR_ORDER,
	C3_NUM_DQ_PINS => C3_NUM_DQ_PINS,
	C3_MEM_ADDR_WIDTH => C3_MEM_ADDR_WIDTH,
	C3_MEM_BANKADDR_WIDTH => C3_MEM_BANKADDR_WIDTH
)
	port map (

	c3_sys_clk	=>		   clk100_i,

	c3_sys_rst_i	=>		 pin_sysreset_i,						
	mcb3_dram_dq	   =>	 mcb3_dram_dq,	
	mcb3_dram_a		   =>	 mcb3_dram_a,  
	mcb3_dram_ba	   =>	 mcb3_dram_ba,
	mcb3_dram_ras_n	   =>	 mcb3_dram_ras_n,						 
	mcb3_dram_cas_n	   =>	 mcb3_dram_cas_n,						 
	mcb3_dram_we_n	   =>	 mcb3_dram_we_n,						  
	mcb3_dram_odt	 =>		 mcb3_dram_odt,
	mcb3_dram_cke	   =>	 mcb3_dram_cke,							 
	mcb3_dram_ck	   =>	 mcb3_dram_ck,							
	mcb3_dram_ck_n	   =>	 mcb3_dram_ck_n,	   
	mcb3_dram_dqs	   =>	 mcb3_dram_dqs,							 
	mcb3_dram_dqs_n	 =>		 mcb3_dram_dqs_n,
	mcb3_dram_udqs	=>		 mcb3_dram_udqs,	-- for X16 parts		   
	mcb3_dram_udqs_n	=>	 mcb3_dram_udqs_n,	-- for X16 parts
	mcb3_dram_udm  =>		 mcb3_dram_udm,		-- for X16 parts
	mcb3_dram_dm  =>	   mcb3_dram_dm,
	c3_clk0	=>			open,
	c3_rst0		=>		  c3_rst0,
	c3_calib_done	   =>	 c3_calib_done,
	mcb3_rzq		=>			  mcb3_rzq,
	mcb3_zio		=>			  mcb3_zio,
	c3_p0_cmd_clk							=>	c3_p0_cmd_clk,
	c3_p0_cmd_en							=>	c3_p0_cmd_en,
	c3_p0_cmd_instr							=>	c3_p0_cmd_instr,
	c3_p0_cmd_bl							=>	c3_p0_cmd_bl,
	c3_p0_cmd_byte_addr						=>	c3_p0_cmd_byte_addr,
	c3_p0_cmd_empty							=>	c3_p0_cmd_empty,
	c3_p0_cmd_full							=>	c3_p0_cmd_full,
	c3_p0_wr_clk							=>	c3_p0_wr_clk,
	c3_p0_wr_en								=>	c3_p0_wr_en,
	c3_p0_wr_mask							=>	c3_p0_wr_mask,
	c3_p0_wr_data							=>	c3_p0_wr_data,
	c3_p0_wr_full							=>	c3_p0_wr_full,
	c3_p0_wr_empty							=>	c3_p0_wr_empty,
	c3_p0_wr_count							=>	c3_p0_wr_count,
	c3_p0_wr_underrun						=>	c3_p0_wr_underrun,
	c3_p0_wr_error							=>	c3_p0_wr_error,
	c3_p0_rd_clk							=>	c3_p0_rd_clk,
	c3_p0_rd_en								=>	c3_p0_rd_en,
	c3_p0_rd_data							=>	c3_p0_rd_data,
	c3_p0_rd_full							=>	c3_p0_rd_full,
	c3_p0_rd_empty							=>	c3_p0_rd_empty,
	c3_p0_rd_count							=>	c3_p0_rd_count,
	c3_p0_rd_overflow						=>	c3_p0_rd_overflow,
	c3_p0_rd_error							=>	c3_p0_rd_error
);

end rtl;
