--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2010 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <Stefan.Fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH CESYS PRODUCTS ONLY!
-- YOU ARE NOT ALLOWED TO MODIFY AND DISTRIBUTE OR USE IT WITH ANY OTHER
-- HARDWARE, SOFTWARE OR ANY OTHER KIND OF ASIC OR PROGRAMMABLE LOGIC DESIGN
-- WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;

use work.wishbone_pkg.all;
use work.wb_sl_flash_pkg.all;


entity wb_sl_flash is
  generic
  (
    NR_OF_DBGPORTS : positive := 1
  );
  port
  (
    syscon_i : in rec_syscon_port;
    master_i : in rec_master_port;
    slave_o : out rec_slave_port;

    flash_s_n_o : out std_logic;
    flash_c_o : out std_logic;
    flash_hold_n_o : out std_logic;
    flash_w_n_o : out std_logic;
    flash_d_o : out std_logic;
    flash_q_i : in std_logic;

    debug : out std_logic_vector((NR_OF_DBGPORTS-1) downto 0)
  );
end wb_sl_flash;


architecture rtl of wb_sl_flash is

  signal syscon : rec_syscon_port := SYSCON_DEFAULT;
  signal master : rec_master_port := MASTER_DEFAULT;
  signal slave : rec_slave_port := SLAVE_DEFAULT;

  signal flash_s_n : std_logic := '1';
  signal flash_c : std_logic := '0';
  signal flash_hold_n : std_logic := '1';
  signal flash_w_n : std_logic := '1';
  signal flash_d : std_logic := '0';
  signal flash_q : std_logic := '0';

  component flash_ctrl is
    generic
    (
      NR_OF_DBGPORTS : positive := 1
    );
    port
    (
      rst : in std_logic;
      clk : in std_logic;
      
      usr_access_i : in std_logic;
      usr_erase_i : in std_logic;
      usr_we_i : in std_logic;
      usr_adr_i : in std_logic_vector(21 downto 0);
      usr_dati_i : in std_logic_vector(31 downto 0);
      usr_dato_o : out std_logic_vector(31 downto 0);
      usr_ok_o : out std_logic;
  
      flash_s_n_o : out std_logic;
      flash_c_o : out std_logic;
      flash_hold_n_o : out std_logic;
      flash_w_n_o : out std_logic;
      flash_d_o : out std_logic;
      flash_q_i : in std_logic;
  
      debug : out std_logic_vector((NR_OF_DBGPORTS-1) downto 0)
    );
  end component;

  signal rst : std_logic := '1';
  signal clk : std_logic := '0';

  signal usr_access : std_logic := '0';
  signal usr_erase : std_logic := '0';
  signal usr_we : std_logic := '0';
  signal usr_adr : std_logic_vector(21 downto 0) := (others  => '0');
  signal usr_dati : std_logic_vector(31 downto 0) := (others  => '0');
  signal usr_dato : std_logic_vector(31 downto 0) := (others  => '0');
  signal usr_ok : std_logic := '0';

begin

  -- connect ports to internal signals
  ------------------------------------
  
  -- ports of WISHBONE bus
  syscon <= syscon_i;
  master <= master_i;
  slave_o <= slave;
  
  -- ports of SPI FLASH
  flash_s_n_o <= flash_s_n;
  flash_c_o <= flash_c;
  flash_hold_n_o <= flash_hold_n;
  flash_w_n_o <= flash_w_n;
  flash_d_o <= flash_d;
  flash_q <= flash_q_i;

  debug <=
  (
    others => '0'
  );

  rst <= syscon.rst;
  clk <= syscon.clk;

  -- wishbone-to-flash controller connections
  -------------------------------------------

  -- applying flash erase command with wishbone write address decoding
  -- decoding only the msb for saving FPGA resources
  -- command register address is one position above flash memory space
 
    usr_dati <= master.dat;
    usr_adr <= master.adr(RNG_FLASH_ADR);        
    usr_we <= master.we;
    usr_erase <= master.cyc and master.stb and master.we and master.adr(RNG_FLASH_ADR'high+1);
    usr_access <= master.cyc and master.stb;
    slave.dat <= usr_dato;
    slave.ack <= usr_ok;  
  
  
  -- instantiate low level flash controller
  -----------------------------------------

  inst_flash_ctrl : flash_ctrl
  generic map
  (
    NR_OF_DBGPORTS => 1
  )
  port map
  (
    rst => rst,
    clk => clk,
  
    usr_access_i => usr_access,
    usr_erase_i => usr_erase,
    usr_we_i => usr_we,
    usr_adr_i => usr_adr,
    usr_dati_i => usr_dati,
    usr_dato_o => usr_dato,
    usr_ok_o => usr_ok,

    flash_s_n_o => flash_s_n,
    flash_c_o => flash_c,
    flash_hold_n_o => flash_hold_n,
    flash_w_n_o => flash_w_n,
    flash_d_o => flash_d,
    flash_q_i => flash_q,

    debug => open
  );

end rtl;
