--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2010 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <Stefan.Fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH CESYS PRODUCTS ONLY!
-- YOU ARE NOT ALLOWED TO MODIFY AND DISTRIBUTE OR USE IT WITH ANY OTHER
-- HARDWARE, SOFTWARE OR ANY OTHER KIND OF ASIC OR PROGRAMMABLE LOGIC DESIGN
-- WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

use work.wishbone_pkg.all;


package wb_sl_uart_pkg is

  -- pragma CONST_EXTRACT_ON
  constant UART_ALIGNMENT : std_logic_vector((ADR_WIDTH-1) downto 0) := x"0000_0004";
  constant UART_RXTX_OFFSET : std_logic_vector((ADR_WIDTH-1) downto 0) := x"0000_0000";
  constant UART_STACFG_OFFSET : std_logic_vector((ADR_WIDTH-1) downto 0) := x"0000_0004";
  subtype UART_RXTX_DATA_FIELD is natural range 7 downto 0;
  constant UART_RXTX_RDVAL_FLAG : natural := 8;
  subtype UART_STACFG_BDR_FIELD is natural range 15 downto 0;
  constant UART_STACFG_RF_FLAG : natural := 31;
  constant UART_STACFG_RHF_FLAG : natural := 30;
  constant UART_STACFG_TF_FLAG : natural := 29;
  constant UART_STACFG_THF_FLAG : natural := 28;
  -- pragma CONST_EXTRACT_OFF
  
  subtype RNG_UART_ADR is natural range 2 downto 2;
  
  component wb_sl_uart is
    generic
    (
      NR_OF_DBGPORTS : positive := 1
    );
    port
    (
      syscon_i : in rec_syscon_port;
      master_i : in rec_master_port;
      slave_o : out rec_slave_port;
      
      uart_rx_i : in std_logic;
      uart_tx_o : out std_logic;

      debug : out std_logic_vector((NR_OF_DBGPORTS-1) downto 0)
    );
  end component;
  
  function BaudCalc(baudrate : real; refclk : real) return std_logic_vector;
  
end wb_sl_uart_pkg;


package body wb_sl_uart_pkg is

  function BaudCalc(baudrate : real; refclk : real) return std_logic_vector is
  begin
    return std_logic_vector(To_Unsigned(natural(Round(refclk/(16.0*baudrate)))-1, 16));
  end BaudCalc;
  
end wb_sl_uart_pkg;
