--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2010 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <Stefan.Fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH CESYS PRODUCTS ONLY!
-- YOU ARE NOT ALLOWED TO MODIFY AND DISTRIBUTE OR USE IT WITH ANY OTHER
-- HARDWARE, SOFTWARE OR ANY OTHER KIND OF ASIC OR PROGRAMMABLE LOGIC DESIGN
-- WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;


entity xil_uart is
  generic
  (
    NR_OF_DBGPORTS : positive := 1
  );
  port
  (
    rst : in std_logic;
    clk : in std_logic;
    
    clk16x_i : in std_logic;

    rx_read_buffer_i : in std_logic;
    rx_data_out_o : out std_logic_vector(7 downto 0);
    rx_buffer_data_present_o : out std_logic;
    rx_buffer_full_o : out std_logic;
    rx_buffer_half_full_o : out std_logic;
    rx_serial_in_i : in std_logic;

    tx_write_buffer_i : in std_logic;
    tx_data_in_i : in std_logic_vector(7 downto 0);
    tx_buffer_full_o : out std_logic;
    tx_buffer_half_full_o : out std_logic;
    tx_serial_out_o : out std_logic;

    debug : out std_logic_vector((NR_OF_DBGPORTS-1) downto 0)
  );
end xil_uart;


architecture macro of xil_uart is

  signal clk16x : std_logic := '0';
  
  component uart_rx is
    port
    (
      serial_in : in std_logic;
      data_out : out std_logic_vector(7 downto 0);
      read_buffer : in std_logic;
      reset_buffer : in std_logic;
      en_16_x_baud : in std_logic;
      buffer_data_present : out std_logic;
      buffer_full : out std_logic;
      buffer_half_full : out std_logic;
      clk : in std_logic
    );
  end component;
  
  signal rx_serial_in : std_logic := '1';
  signal rx_data_out : std_logic_vector(7 downto 0) := (others => '0');
  signal rx_read_buffer : std_logic := '0';
  signal rx_reset_buffer : std_logic := '1';
  signal rx_en_16_x_baud : std_logic := '0';
  signal rx_buffer_data_present : std_logic := '0';
  signal rx_buffer_full : std_logic := '0';
  signal rx_buffer_half_full : std_logic := '0';
  signal rx_clk : std_logic := '0';

  component uart_tx is
    port
    (
      data_in : in std_logic_vector(7 downto 0);
      write_buffer : in std_logic;
      reset_buffer : in std_logic;
      en_16_x_baud : in std_logic;
      serial_out : out std_logic;
      buffer_full : out std_logic;
      buffer_half_full : out std_logic;
      clk : in std_logic
    );
  end component;
  
  signal tx_data_in : std_logic_vector(7 downto 0) := (others => '0');
  signal tx_write_buffer : std_logic := '0';
  signal tx_reset_buffer : std_logic := '1';
  signal tx_en_16_x_baud : std_logic := '0';
  signal tx_serial_out : std_logic := '1';
  signal tx_buffer_full : std_logic := '0';
  signal tx_buffer_half_full : std_logic := '0';
  signal tx_clk : std_logic := '0';
  
begin

  -- connect ports to internal signals
  ------------------------------------

  clk16x <= clk16x_i;
  
  rx_read_buffer <= rx_read_buffer_i;
  rx_data_out_o <= rx_data_out;
  rx_buffer_data_present_o <= rx_buffer_data_present;
  rx_buffer_full_o <= rx_buffer_full;
  rx_buffer_half_full_o <= rx_buffer_half_full;
  rx_serial_in <= rx_serial_in_i;

  tx_write_buffer <= tx_write_buffer_i;
  tx_data_in <= tx_data_in_i;
  tx_buffer_full_o <= tx_buffer_full;
  tx_buffer_half_full_o <= tx_buffer_half_full;
  tx_serial_out_o <= tx_serial_out;

  debug <=
  (
    others => '0'
  );
  
  rx_reset_buffer <= rst;
  rx_clk <= clk;
  rx_en_16_x_baud <= clk16x;

  inst_uart_rx : uart_rx
  port map
  (
    serial_in => rx_serial_in,
    data_out => rx_data_out,
    read_buffer => rx_read_buffer,
    reset_buffer => rx_reset_buffer,
    en_16_x_baud => rx_en_16_x_baud,
    buffer_data_present => rx_buffer_data_present,
    buffer_full => rx_buffer_full,
    buffer_half_full => rx_buffer_half_full,
    clk => rx_clk
  );
  
  tx_reset_buffer <= rst;
  tx_clk <= clk;
  tx_en_16_x_baud <= clk16x;

  inst_uart_tx : uart_tx
  port map
  (
    data_in => tx_data_in,
    write_buffer => tx_write_buffer,
    reset_buffer => tx_reset_buffer,
    en_16_x_baud => tx_en_16_x_baud,
    serial_out => tx_serial_out,
    buffer_full => tx_buffer_full,
    buffer_half_full => tx_buffer_half_full,
    clk => tx_clk
  );
    
end macro;
